/***************************************************************************
    copyright            : (C) 2005 by Brian Kerrick Nickel
    email                : brian.nickel@gmail.com
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or modify  *
 *   it  under the terms of the GNU Lesser General Public License version  *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful, but   *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
 *   USA                                                                   *
 ***************************************************************************/

#include <string.h>

#include "StringList.h"


struct _StringList
{
   TagLib::StringList::ConstIterator it;
   TagLib::StringList * sl;
};

StringList * string_list_from_string (TagLib::String str)
{
   StringList * sl = (StringList *) malloc (sizeof (StringList));
   sl->sl = new TagLib::StringList (TagLib::StringList::split (str, ";"));
   sl->it = NULL;
   return sl;
}


StringList * string_list_from_field (const TagLib::StringList stl)
{
   StringList * sl = (StringList *) malloc (sizeof (StringList));
   sl->sl = new TagLib::StringList (stl);
   sl->it = NULL;
   return sl;
}


extern "C" StringList * string_list_new (char * s)
{
   return string_list_from_string (TagLib::String(s, TagLib::String::UTF8));
}


extern "C" void string_list_add_string (StringList * sl, char * str)
{
   if (sl == NULL)
      return;
   sl->sl->append (TagLib::String(str, TagLib::String::UTF8));
}


extern "C" char * string_list_get_first (StringList * sl)
{
   if (!string_list_get_length(sl))
      return NULL;
   sl->it = sl->sl->begin ();
   return ::strdup((*(sl->it)).toCString (1));
}


extern "C" char * string_list_get_next (StringList * sl)
{
   if (sl == NULL || sl->it == NULL)
      return string_list_get_first (sl);
   ++(sl->it);
   if (sl->it == sl->sl->end ())
   {
      sl->it = NULL;
      return NULL;
   }
   TagLib::String str = *(sl->it);
   return ::strdup(str.toCString (1));
}


extern "C" uint string_list_get_length (StringList * sl)
{
   return (sl == NULL) ? 0 : sl->sl->size ();
}


extern "C" void string_list_free (StringList * sl)
{
   if (sl == NULL)
      return;
   if (sl->sl)
      delete sl->sl;
   free (sl);
}


TagLib::String string_list_to_string (StringList * sl)
{
   if (!sl || !sl->sl)
      return "";
   return sl->sl->toString ("; ");
}

TagLib::StringList string_list_get (StringList * sl)
{
   return *(sl->sl);
}
