/***************************************************************************
    copyright            : (C) 2005 by Brian Kerrick Nickel
    email                : brian.nickel@gmail.com
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or modify  *
 *   it  under the terms of the GNU Lesser General Public License version  *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful, but   *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
 *   USA                                                                   *
 ***************************************************************************/

#ifndef __TAGDATA_H
#define __TAGDATA_H

#include <taglib.h>
#include <id3v2tag.h>
#include <id3v2frame.h>
#include <id3v2header.h>
#include <id3v1tag.h>
#include <attachedpictureframe.h>
#include <textidentificationframe.h>
#include <relativevolumeframe.h>
#include <commentsframe.h>
#include <audioproperties.h>
#include <apetag.h>
#include <xiphcomment.h>
#include <vorbisfile.h>
#include <mpegfile.h>
#include <flacfile.h>
#include <mpcfile.h>
#include <fileref.h>
#include <tag.h>

#include <string.h>

typedef struct _TagData
{
   TagLib::FileRef          * file;
   TagLib::ID3v2::Tag       * id3v2;
   TagLib::ID3v1::Tag       * id3v1;
   TagLib::APE::Tag         * ape;
   TagLib::Ogg::XiphComment * xiph;
   char                     * mime;
   int                        mtime;
} TagData;

#include "StringList.h"
#include "framehandlers.h"

extern "C" TagData * tag_data_load (char *filename, int scanaudio);
extern "C" bool tag_data_save (TagData * data);
extern "C" void tag_data_free (TagData * data);


// STRING FUNCTIONS

extern "C" const char * tag_data_get_title (TagData * data);
extern "C" void tag_data_set_title (TagData * data, char * val);

extern "C" const char * tag_data_get_album (TagData * data);
extern "C" void tag_data_set_album (TagData * data, char * val);

extern "C" char * tag_data_get_disc_name (TagData * data);
extern "C" void tag_data_set_disc_name (TagData * data, char * str);

extern "C" const char * tag_data_get_comment (TagData * data);
extern "C" void tag_data_set_comment (TagData * data, char * val);

// STRING LIST FUNCTIONS

extern "C" StringList * tag_data_get_artists (TagData * data);
extern "C" void tag_data_set_artists (TagData * data, StringList * val);

extern "C" StringList * tag_data_get_genre (TagData * data);
extern "C" void  tag_data_set_genre (TagData * data, StringList * val);

extern "C" StringList * tag_data_get_performers (TagData * data);
extern "C" void tag_data_set_performers (TagData * data, StringList * val);

extern "C" StringList * tag_data_get_mood (TagData * data);
extern "C" void tag_data_set_mood (TagData * data, StringList * val);

// NUMBER FUNCTIONS

extern "C" uint tag_data_get_year (TagData * data);
extern "C" void tag_data_set_year (TagData * data, uint val);

extern "C" uint tag_data_get_track (TagData * data);
extern "C" uint tag_data_get_track_count (TagData * data);
extern "C" void tag_data_set_track (TagData * data, uint track, uint track_count);

extern "C" uint tag_data_get_disc_number (TagData * data);
extern "C" uint tag_data_get_disc_count (TagData * data);
extern "C" void tag_data_set_disc_number (TagData * data, uint disc, uint disc_count);


extern "C" uint tag_data_get_duration (TagData * data);

extern "C" float tag_data_get_gain (TagData * data);
extern "C" void tag_data_set_gain (TagData * data, float gain);

extern "C" float tag_data_get_peak (TagData * data);

extern "C" const char * tag_data_get_mime (TagData * data);

extern "C" int tag_data_get_mtime (TagData * data);

#endif
