/***************************************************************************
    copyright            : (C) 2005 by Brian Kerrick Nickel
    email                : brian.nickel@gmail.com
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or modify  *
 *   it  under the terms of the GNU Lesser General Public License version  *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful, but   *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
 *   USA                                                                   *
 ***************************************************************************/

#include "framehandlers.h"

/*TSTRING
 */
//GET
TagLib::String tag_data_get_xiph_tstring (TagData * data, const char * frame)
{
   if (data->xiph && !data->xiph->fieldListMap()[frame].isEmpty())
      return data->xiph->fieldListMap()[frame].front();
   return "";
}
TagLib::String tag_data_get_ape_tstring (TagData * data, const char * frame)
{
   if (data->ape && !data->ape->itemListMap()[frame].isEmpty())
      return data->ape->itemListMap()[frame].toString();
   return "";
}
TagLib::String tag_data_get_id3v2_tstring (TagData * data, const char * frame)
{
   if (data->id3v2 && !data->id3v2->frameListMap()[frame].isEmpty())
      return data->id3v2->frameListMap()[frame].front()->toString();
   return "";
}
TagLib::String tag_data_get_frame_tstring (TagData * data, const char * com, const char * id3)
{
   TagLib::String str;
   str = tag_data_get_id3v2_tstring (data, id3);
   if (!(str == "")) return str;
   str = tag_data_get_xiph_tstring (data, com);
   if (!(str == "")) return str;
   str = tag_data_get_ape_tstring (data, com);
   return str;
}
//SET
void tag_data_set_xiph_tstring (TagData * data, const char * frame, TagLib::String str)
{
   if (data->xiph)
      (str == "") ? data->xiph->removeField(frame) : data->xiph->addField(frame, str);
}
void tag_data_set_ape_tstring (TagData * data, const char * frame, TagLib::String str)
{
   if (data->ape)
      (str == "") ? data->ape->removeItem(frame) : data->ape->addValue(frame, str, true);
}
void tag_data_set_id3v2_tstring (TagData * data, const char * frame, TagLib::String str, int utf8)
{
   if (!data->id3v2)
      return;
   
   if (str == "")
      data->id3v2->removeFrames(frame);
   else
   {
      TagLib::ID3v2::TextIdentificationFrame * text;
      if (data->id3v2->frameListMap()[frame].isEmpty ())
      {
         text = new TagLib::ID3v2::TextIdentificationFrame(frame, TagLib::String::UTF8);
         data->id3v2->addFrame (text);
      }
      else
         text = dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*> (data->id3v2->frameListMap()[frame].front ());
      text->setText (str);
      text->setTextEncoding (utf8 ? TagLib::String::UTF8 : TagLib::String::Latin1);
   }
}
void tag_data_set_frame_tstring (TagData * data, const char * com, const char * id3, TagLib::String str)
{
   tag_data_set_id3v2_tstring (data, id3, str, 1);
   tag_data_set_xiph_tstring (data, com, str);
   tag_data_set_ape_tstring (data, com, str);
}



/*CSTRING
 */
//GET
char * tag_data_get_xiph_string (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_xiph_tstring (data, frame);
   return ::strdup(str.toCString (1));
}
char * tag_data_get_ape_string (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_ape_tstring (data, frame);
   return ::strdup(str.toCString (1));
}
char * tag_data_get_id3v2_string (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_id3v2_tstring (data, frame);
   return ::strdup(str.toCString (1));
}
char * tag_data_get_frame_string (TagData * data, const char * com, const char * id3)
{
   TagLib::String str = tag_data_get_frame_tstring (data, com, id3);
   return ::strdup(str.toCString (1));
}
//SET
void tag_data_set_xiph_string (TagData * data, const char * frame, char * str)
{
   tag_data_set_xiph_tstring (data, frame, (str && str[0]) ? TagLib::String (str, TagLib::String::UTF8) : "");
}
void tag_data_set_ape_string (TagData * data, const char * frame, char * str)
{
   tag_data_set_ape_tstring (data, frame, (str && str[0]) ? TagLib::String (str, TagLib::String::UTF8) : "");
}
void tag_data_set_id3v2_string (TagData * data, const char * frame, char * str, int utf8)
{
   tag_data_set_id3v2_tstring (data, frame, (str && str[0]) ? TagLib::String (str, TagLib::String::UTF8) : "", 0);
}
void tag_data_set_frame_string (TagData * data, const char * com, const char * id3, char * str)
{
   tag_data_set_frame_tstring (data, com, id3, (str && str[0]) ? TagLib::String (str, TagLib::String::UTF8) : "");
}



/*UINT
 */
//GET
uint tag_data_get_xiph_uint (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_xiph_tstring (data, frame);
   return (str == "") ? 0 : (uint) str.toInt ();
}
uint tag_data_get_ape_uint (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_ape_tstring (data, frame);
   return (str == "") ? 0 : (uint) str.toInt ();
}
uint tag_data_get_id3v2_uint (TagData * data, const char * frame)
{
   TagLib::String str = tag_data_get_id3v2_tstring (data, frame);
   return (str == "") ? 0 : (uint) str.toInt ();
}
uint tag_data_get_frame_uint (TagData * data, const char * com, const char * id3)
{
   TagLib::String str = tag_data_get_frame_tstring (data, com, id3);
   return (str == "") ? 0 : (uint) str.toInt ();
}
//SET
void tag_data_set_xiph_uint (TagData * data, const char * frame, uint num)
{
   tag_data_set_xiph_tstring (data, frame, (num) ? TagLib::String::number(num) : "");
}
void tag_data_set_ape_uint (TagData * data, const char * frame, uint num)
{
   tag_data_set_ape_tstring (data, frame, (num) ? TagLib::String::number(num) : "");
}
void tag_data_set_id3v2_uint (TagData * data, const char * frame, uint num)
{
   tag_data_set_id3v2_tstring (data, frame, (num) ? TagLib::String::number(num) : "", 0);
}
void tag_data_set_frame_uint (TagData * data, const char * com, const char * id3, uint num)
{
   tag_data_set_frame_tstring (data, com, id3, (num) ? TagLib::String::number(num) : "");
}



/*STRINGS
 */
//GET
StringList * tag_data_get_xiph_strings (TagData * data, const char * frame)
{
   StringList * sl = NULL;
   if (data->xiph && !data->xiph->fieldListMap()[frame].isEmpty())
   {
      TagLib::StringList::ConstIterator it;
      TagLib::StringList l = data->xiph->fieldListMap()[frame];
      for(it=l.begin(); it!=l.end(); ++it)
         if (sl == NULL)
            sl = string_list_new ((char *)(*it).toCString (1));
         else
            string_list_add_string (sl, (char *)(*it).toCString (1));
   }
   return sl;
}
StringList * tag_data_get_ape_strings (TagData * data, const char * frame)
{
   StringList * sl = NULL;
   if (data->ape && !data->ape->itemListMap()[frame].isEmpty())
   {
      TagLib::StringList::ConstIterator it;
      TagLib::StringList l = data->xiph->fieldListMap()[frame];
      for(it=l.begin(); it!=l.end(); ++it)
         if (sl == NULL)
            sl = string_list_new ((char *)(*it).toCString (1));
         else
            string_list_add_string (sl, (char *)(*it).toCString (1));
   }
   return sl;
}
StringList * tag_data_get_id3v2_strings (TagData * data, const char * frame)
{
   if (data->id3v2 && !data->id3v2->frameListMap()[frame].isEmpty())
   {
      TagLib::ID3v2::TextIdentificationFrame * text = dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*> (data->id3v2->frameListMap()[frame].front());
      return string_list_from_field (text->fieldList ());
   }
   return NULL;
}
StringList * tag_data_get_frame_strings (TagData * data, const char * com, const char * id3)
{
   StringList * sl;
   sl = tag_data_get_id3v2_strings (data, id3);
   if (sl) return sl;
   sl = tag_data_get_xiph_strings (data, com);
   if (sl) return sl;
   sl = tag_data_get_ape_strings (data, com);
   return sl;
}
//SET
void tag_data_set_xiph_strings (TagData * data, const char * frame, StringList * sl)
{
   if (!data->xiph)
      return;
   if (!string_list_get_length(sl))
   {
      data->xiph->removeField(frame);
      return;
   }
   
   data->xiph->addField(frame, string_list_get_first (sl), true);
   char * str;
   while (str = string_list_get_next (sl))
      data->xiph->addField (frame, str, false);
}
void tag_data_set_ape_strings (TagData * data, const char * frame, StringList * sl)
{
   if (!data->ape)
      return;
   if (!string_list_get_length(sl))
   {
      data->ape->removeItem(frame);
      return;
   }
   
   data->ape->addValue(frame, string_list_get_first (sl), true);
   char * str;
   while (str = string_list_get_next (sl))
      data->ape->addValue (frame, str, false);
}
void tag_data_set_id3v2_strings (TagData * data, const char * frame, StringList * sl, int utf8)
{
   if (!data->id3v2)
      return;
   
   if (sl && string_list_get_length(sl))
   {
      TagLib::ID3v2::TextIdentificationFrame * text;
      if (data->id3v2->frameListMap()[frame].isEmpty ())
      {
         text = new TagLib::ID3v2::TextIdentificationFrame(frame, TagLib::String::UTF8);
         data->id3v2->addFrame (text);
      }
      else
         text = dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*> (data->id3v2->frameListMap()[frame].front ());
      text->setText (string_list_get (sl));
      text->setTextEncoding (utf8 ? TagLib::String::UTF8 : TagLib::String::Latin1);
   }
   else
      data->id3v2->removeFrames (frame);
}
void tag_data_set_frame_strings (TagData * data, const char * com, const char * id3, StringList * sl)
{
   tag_data_set_id3v2_strings (data, id3, sl, 1);
   tag_data_set_xiph_strings (data, com, sl);
   tag_data_set_ape_strings (data, com, sl);
}
